﻿using System;
using System.Windows.Forms;

using Emgu.CV;
using Emgu.CV.CvEnum;
using Emgu.CV.Stitching;
using Emgu.CV.Structure;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 파일 열기 대화 상자
        /// </summary>
        private OpenFileDialog openFileDialog;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.openFileDialog = new OpenFileDialog();

            this.openFileDialog.CheckFileExists = true;
            this.openFileDialog.Multiselect     = true;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 이미지 선택 버튼 클릭시 처리하기 - selectButton_Click(sender, e)

        /// <summary>
        /// 이미지 선택 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void selectButton_Click(object sender, EventArgs e)
        {
            if(this.openFileDialog.ShowDialog() == DialogResult.OK)
            {
                this.imageDataGridView.Rows.Clear();

                Image<Bgr, Byte>[] sourceImageArray = new Image<Bgr, byte>[this.openFileDialog.FileNames.Length];
            
                int sourceImageArrayLength = sourceImageArray.Length;

                for(int i = 0; i < sourceImageArrayLength; i++)
                {
                    sourceImageArray[i] = new Image<Bgr, byte>(this.openFileDialog.FileNames[i]);

                    using(Image<Bgr, byte> thumbnailImage = sourceImageArray[i].Resize(200, 200, INTER.CV_INTER_CUBIC, true))
                    {
                        DataGridViewRow row = this.imageDataGridView.Rows[imageDataGridView.Rows.Add()];

                        row.Cells["fileNameColumn" ].Value = this.openFileDialog.FileNames[i];
                        row.Cells["thumbnailColumn"].Value = thumbnailImage.ToBitmap();

                        row.Height = 200;
                    }
                }

                try
                {
                    using(Stitcher stitcher = new Stitcher(true))
                    {
                        Image<Bgr, Byte> resultImage = stitcher.Stitch(sourceImageArray);

                        this.resultImageBox.Image = resultImage;
                    }
                }
                finally
                {
                    foreach(Image<Bgr, Byte> sourceImage in sourceImageArray)
                    {
                        sourceImage.Dispose();
                    }
                }
            }
        }

        #endregion
    }
}